@@ -59,12 +59,13 @@
 #include "dfa_min.h"
 
 #include "grey.h"
+#include "mcclellancompile_util.h"
 #include "rdfa.h"
 #include "ue2common.h"
 #include "util/container.h"
+#include "util/flat_containers.h"
 #include "util/noncopyable.h"
 #include "util/partitioned_set.h"
-#include "util/ue2_containers.h"
 
 #include <algorithm>
 #include <functional>
@@ -299,6 +300,10 @@ void minimize_hopcroft(raw_dfa &rdfa, const Grey &grey) {
         return;
     }
 
+    if (is_dead(rdfa)) {
+        DEBUG_PRINTF("dfa is empty\n");
+    }
+
     UNUSED const size_t states_before = rdfa.states.size();
 
     HopcroftInfo info(rdfa);
