@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2016, Intel Corporation
+ * Copyright (c) 2015-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -45,8 +45,6 @@
 #include <memory>
 #include <sstream>
 
-#include <boost/graph/depth_first_search.hpp>
-
 using namespace std;
 
 namespace ue2 {
@@ -194,7 +192,7 @@ struct PathMask {
     }
 
     vector<CharReach> mask;
-    ue2::flat_set<ReportID> reports;
+    flat_set<ReportID> reports;
     bool is_anchored;
     bool is_eod;
 };
@@ -210,6 +208,11 @@ bool handleDecoratedLiterals(RoseBuild &rose, const NGHolder &g,
         return false;
     }
 
+    if (!hasNarrowReachVertex(g)) {
+        DEBUG_PRINTF("no narrow reach vertices\n");
+        return false;
+    }
+
     if (hasLargeDegreeVertex(g)) {
         DEBUG_PRINTF("large degree\n");
         return false;
