@@ -39,7 +39,6 @@
 #include "ng_limex.h"
 #include "ng_redundancy.h"
 #include "ng_region.h"
-#include "ng_restructuring.h"
 #include "ng_uncalc_components.h"
 #include "ng_util.h"
 #include "ue2common.h"
@@ -55,42 +54,52 @@
 #include <set>
 #include <vector>
 
+#include <boost/range/adaptor/map.hpp>
+
 using namespace std;
+using boost::adaptors::map_values;
 
 namespace ue2 {
 
 static const u32 FAST_STATE_LIMIT = 256; /**< largest possible desirable NFA */
 
 /** Sentinel value meaning no component has yet been selected. */
-static const u32 NO_COMPONENT = 0xffffffffu;
+static const u32 NO_COMPONENT = ~0U;
 
-static
-vector<NFAVertex> getSortedVA(const NGHolder &g,
-            const ue2::unordered_map<NFAVertex, u32> &state_ids) {
-    vector<NFAVertex> out;
-    out.reserve(num_vertices(g));
+static const u32 UNUSED_STATE = ~0U;
 
-    for (auto v : vertices_range(g)) {
-        assert(contains(state_ids, v));
-        if (state_ids.at(v) == NO_STATE) {
-            continue;
+namespace {
+struct ranking_info {
+    explicit ranking_info(const NGHolder &h) : to_vertex(getTopoOrdering(h)) {
+        u32 rank = 0;
+
+        reverse(to_vertex.begin(), to_vertex.end());
+
+        for (NFAVertex v : to_vertex) {
+            to_rank[v] = rank++;
+        }
+
+        for (NFAVertex v : vertices_range(h)) {
+            if (!contains(to_rank, v)) {
+                to_rank[v] = UNUSED_STATE;
+            }
         }
-        out.push_back(v);
     }
 
-    // Order vertices by their state indices.
-    sort(begin(out), end(out), [&state_ids](NFAVertex a, NFAVertex b) {
-        return state_ids.at(a) < state_ids.at(b);
-    });
-
-#ifndef NDEBUG
-    // State indices should match vector indices.
-    for (u32 i = 0; i < out.size(); i++) {
-        assert(state_ids.at(out.at(i)) == i);
+    NFAVertex at(u32 ranking) const { return to_vertex.at(ranking); }
+    u32 get(NFAVertex v) const { return to_rank.at(v); }
+    u32 size() const { return (u32)to_vertex.size(); }
+    u32 add_to_tail(NFAVertex v) {
+        u32 rank = size();
+        to_rank[v] = rank;
+        to_vertex.push_back(v);
+        return rank;
     }
-#endif
 
-    return out;
+private:
+    vector<NFAVertex> to_vertex;
+    unordered_map<NFAVertex, u32> to_rank;
+};
 }
 
 static never_inline
@@ -122,9 +131,9 @@ bool cplVerticesMatch(const NGHolder &ga, NFAVertex va,
 }
 
 static never_inline
-u32 cplCommonReachAndSimple(const NGHolder &ga, const vector<NFAVertex> &a,
-                            const NGHolder &gb, const vector<NFAVertex> &b) {
-    u32 ml = min(a.size(), b.size());
+u32 cplCommonReachAndSimple(const NGHolder &ga, const ranking_info &a_ranking,
+                            const NGHolder &gb, const ranking_info &b_ranking) {
+    u32 ml = min(a_ranking.size(), b_ranking.size());
     if (ml > 65535) {
         ml = 65535;
     }
@@ -133,7 +142,7 @@ u32 cplCommonReachAndSimple(const NGHolder &ga, const vector<NFAVertex> &a,
     // "startedness" properties.
     u32 max = 0;
     for (; max < ml; max++) {
-        if (!cplVerticesMatch(ga, a[max], gb, b[max])) {
+        if (!cplVerticesMatch(ga, a_ranking.at(max), gb, b_ranking.at(max))) {
             break;
         }
     }
@@ -141,34 +150,30 @@ u32 cplCommonReachAndSimple(const NGHolder &ga, const vector<NFAVertex> &a,
     return max;
 }
 
-u32 commonPrefixLength(const NGHolder &ga,
-                       const ue2::unordered_map<NFAVertex, u32> &a_state_ids,
-                       const NGHolder &gb,
-                       const ue2::unordered_map<NFAVertex, u32> &b_state_ids) {
-    vector<NFAVertex> a = getSortedVA(ga, a_state_ids);
-    vector<NFAVertex> b = getSortedVA(gb, b_state_ids);
-
+static
+u32 commonPrefixLength(const NGHolder &ga, const ranking_info &a_ranking,
+                       const NGHolder &gb, const ranking_info &b_ranking) {
     /* upper bound on the common region based on local properties */
-    u32 max = cplCommonReachAndSimple(ga, a, gb, b);
+    u32 max = cplCommonReachAndSimple(ga, a_ranking, gb, b_ranking);
     DEBUG_PRINTF("cpl upper bound %u\n", max);
 
     while (max > 0) {
-        bool ok = true;
-
         /* shrink max region based on in-edges from outside the region */
         for (size_t j = max; j > 0; j--) {
-            for (auto u : inv_adjacent_vertices_range(a[j - 1], ga)) {
-                u32 state_id = a_state_ids.at(u);
-                if (state_id != NO_STATE && state_id >= max) {
+            NFAVertex a_v = a_ranking.at(j - 1);
+            NFAVertex b_v = b_ranking.at(j - 1);
+            for (auto u : inv_adjacent_vertices_range(a_v, ga)) {
+                u32 state_id = a_ranking.get(u);
+                if (state_id != UNUSED_STATE && state_id >= max) {
                     max = j - 1;
                     DEBUG_PRINTF("lowering max to %u\n", max);
                     goto next_vertex;
                 }
             }
 
-            for (auto u : inv_adjacent_vertices_range(b[j - 1], gb)) {
-                u32 state_id = b_state_ids.at(u);
-                if (state_id != NO_STATE && state_id >= max) {
+            for (auto u : inv_adjacent_vertices_range(b_v, gb)) {
+                u32 state_id = b_ranking.get(u);
+                if (state_id != UNUSED_STATE && state_id >= max) {
                     max = j - 1;
                     DEBUG_PRINTF("lowering max to %u\n", max);
                     goto next_vertex;
@@ -180,44 +185,37 @@ u32 commonPrefixLength(const NGHolder &ga,
 
         /* Ensure that every pair of vertices has same out-edges to vertices in
            the region. */
-        for (size_t i = 0; ok && i < max; i++) {
+        for (size_t i = 0; i < max; i++) {
             size_t a_count = 0;
             size_t b_count = 0;
 
-            NGHolder::out_edge_iterator ei, ee;
-            for (tie(ei, ee) = out_edges(a[i], ga); ok && ei != ee; ++ei) {
-                u32 sid = a_state_ids.at(target(*ei, ga));
-                if (sid == NO_STATE || sid >= max) {
+            for (NFAEdge a_edge : out_edges_range(a_ranking.at(i), ga)) {
+                u32 sid = a_ranking.get(target(a_edge, ga));
+                if (sid == UNUSED_STATE || sid >= max) {
                     continue;
                 }
 
                 a_count++;
 
-                NFAEdge b_edge;
-                bool has_b_edge;
-                tie(b_edge, has_b_edge) = edge(b[i], b[sid], gb);
+                NFAEdge b_edge = edge(b_ranking.at(i), b_ranking.at(sid), gb);
 
-                if (!has_b_edge) {
+                if (!b_edge) {
                     max = i;
-                    ok = false;
                     DEBUG_PRINTF("lowering max to %u due to edge %zu->%u\n",
                                  max, i, sid);
-                    break;
+                    goto try_smaller;
                 }
 
-                if (ga[*ei].top != gb[b_edge].top) {
+                if (ga[a_edge].tops != gb[b_edge].tops) {
                     max = i;
-                    ok = false;
-                    DEBUG_PRINTF("tops don't match on edge %zu->%u\n",
-                                 i, sid);
+                    DEBUG_PRINTF("tops don't match on edge %zu->%u\n", i, sid);
+                    goto try_smaller;
                 }
             }
 
-            NGHolder::adjacency_iterator ai, ae;
-            for (tie(ai, ae) = adjacent_vertices(b[i], gb); ok && ai != ae;
-                 ++ai) {
-                u32 sid = b_state_ids.at(*ai);
-                if (sid == NO_STATE || sid >= max) {
+            for (NFAVertex b_v : adjacent_vertices_range(b_ranking.at(i), gb)) {
+                u32 sid = b_ranking.get(b_v);
+                if (sid == UNUSED_STATE || sid >= max) {
                     continue;
                 }
 
@@ -226,52 +224,54 @@ u32 commonPrefixLength(const NGHolder &ga,
 
             if (a_count != b_count) {
                 max = i;
-                DEBUG_PRINTF("lowering max to %u due to a,b count "
-                             "(a_count=%zu, b_count=%zu)\n", max, a_count,
-                             b_count);
-                ok = false;
+                DEBUG_PRINTF("lowering max to %u due to a,b count (a_count=%zu,"
+                             " b_count=%zu)\n", max, a_count, b_count);
+                goto try_smaller;
             }
         }
 
-        if (ok) {
-            DEBUG_PRINTF("survived checks, returning cpl %u\n", max);
-            return max;
-        }
+        DEBUG_PRINTF("survived checks, returning cpl %u\n", max);
+        return max;
+    try_smaller:;
     }
 
     DEBUG_PRINTF("failed to find any common region\n");
     return 0;
 }
 
+u32 commonPrefixLength(const NGHolder &ga, const NGHolder &gb) {
+    return commonPrefixLength(ga, ranking_info(ga), gb, ranking_info(gb));
+}
+
 static never_inline
-void mergeNfa(NGHolder &dest, vector<NFAVertex> &destStateMap,
-              ue2::unordered_map<NFAVertex, u32> &dest_state_ids,
-              NGHolder &vic, vector<NFAVertex> &vicStateMap,
-              size_t common_len) {
+void mergeNfaComponent(NGHolder &dest, const NGHolder &vic, size_t common_len) {
+    assert(&dest != &vic);
+
+    auto dest_info = ranking_info(dest);
+    auto vic_info = ranking_info(vic);
+
     map<NFAVertex, NFAVertex> vmap; // vic -> dest
 
     vmap[vic.start]     = dest.start;
     vmap[vic.startDs]   = dest.startDs;
     vmap[vic.accept]    = dest.accept;
     vmap[vic.acceptEod] = dest.acceptEod;
-    vmap[nullptr] = nullptr;
-
-    u32 stateNum = countStates(dest, dest_state_ids);
+    vmap[NGHolder::null_vertex()] = NGHolder::null_vertex();
 
     // For vertices in the common len, add to vmap and merge in the reports, if
     // any.
     for (u32 i = 0; i < common_len; i++) {
-        NFAVertex v_old = vicStateMap[i], v = destStateMap[i];
+        NFAVertex v_old = vic_info.at(i);
+        NFAVertex v = dest_info.at(i);
         vmap[v_old] = v;
 
         const auto &reports = vic[v_old].reports;
         dest[v].reports.insert(reports.begin(), reports.end());
     }
 
-    // Add in vertices beyond the common len, giving them state numbers
-    // starting at stateNum.
-    for (u32 i = common_len; i < vicStateMap.size(); i++) {
-        NFAVertex v_old = vicStateMap[i];
+    // Add in vertices beyond the common len
+    for (u32 i = common_len; i < vic_info.size(); i++) {
+        NFAVertex v_old = vic_info.at(i);
 
         if (is_special(v_old, vic)) {
             // Dest already has start vertices, just merge the reports.
@@ -283,15 +283,17 @@ void mergeNfa(NGHolder &dest, vector<NFAVertex> &destStateMap,
         }
 
         NFAVertex v = add_vertex(vic[v_old], dest);
-        dest_state_ids[v] = stateNum++;
+        dest_info.add_to_tail(v);
         vmap[v_old] = v;
     }
 
     /* add edges */
     DEBUG_PRINTF("common_len=%zu\n", common_len);
     for (const auto &e : edges_range(vic)) {
-        NFAVertex u_old = source(e, vic), v_old = target(e, vic);
-        NFAVertex u = vmap[u_old], v = vmap[v_old];
+        NFAVertex u_old = source(e, vic);
+        NFAVertex v_old = target(e, vic);
+        NFAVertex u = vmap[u_old];
+        NFAVertex v = vmap[v_old];
         bool uspecial = is_special(u, dest);
         bool vspecial = is_special(v, dest);
 
@@ -302,15 +304,14 @@ void mergeNfa(NGHolder &dest, vector<NFAVertex> &destStateMap,
 
         // We're in the common region if v's state ID is low enough, unless v
         // is a special (an accept), in which case we use u's state ID.
-        assert(contains(dest_state_ids, v));
-        bool in_common_region = dest_state_ids.at(v) < common_len;
-        if (vspecial && dest_state_ids.at(u) < common_len) {
+        bool in_common_region = dest_info.get(v) < common_len;
+        if (vspecial && dest_info.get(u) < common_len) {
             in_common_region = true;
         }
 
-        DEBUG_PRINTF("adding idx=%u (state %u) -> idx=%u (state %u)%s\n",
-                     dest[u].index, dest_state_ids.at(u),
-                     dest[v].index, dest_state_ids.at(v),
+        DEBUG_PRINTF("adding idx=%zu (state %u) -> idx=%zu (state %u)%s\n",
+                     dest[u].index, dest_info.get(u),
+                     dest[v].index, dest_info.get(v),
                      in_common_region ? " [common]" : "");
 
         if (in_common_region) {
@@ -318,7 +319,7 @@ void mergeNfa(NGHolder &dest, vector<NFAVertex> &destStateMap,
                 DEBUG_PRINTF("skipping common edge\n");
                 assert(edge(u, v, dest).second);
                 // Should never merge edges with different top values.
-                assert(vic[e].top == dest[edge(u, v, dest).first].top);
+                assert(vic[e].tops == dest[edge(u, v, dest)].tops);
                 continue;
             } else {
                 assert(is_any_accept(v, dest));
@@ -334,20 +335,8 @@ void mergeNfa(NGHolder &dest, vector<NFAVertex> &destStateMap,
         add_edge(u, v, vic[e], dest);
     }
 
-    dest.renumberEdges();
-    dest.renumberVertices();
-}
-
-static never_inline
-void mergeNfaComponent(NGHolder &pholder, NGHolder &vholder, size_t cpl) {
-    assert(&pholder != &vholder);
-
-    auto v_state_ids = numberStates(vholder);
-    auto p_state_ids = numberStates(pholder);
-    auto vhvmap = getSortedVA(vholder, v_state_ids);
-    auto phvmap = getSortedVA(pholder, p_state_ids);
-
-    mergeNfa(pholder, phvmap, p_state_ids, vholder, vhvmap, cpl);
+    renumber_edges(dest);
+    renumber_vertices(dest);
 }
 
 namespace {
@@ -374,14 +363,19 @@ struct NfaMergeCandidateH {
 
 /** Returns true if graphs \p h1 and \p h2 can (and should) be merged. */
 static
-bool shouldMerge(NGHolder &ha,
-                 const ue2::unordered_map<NFAVertex, u32> &a_state_ids,
-                 NGHolder &hb,
-                 const ue2::unordered_map<NFAVertex, u32> &b_state_ids,
-                 size_t cpl, const ReportManager *rm,
-                 const CompileContext &cc) {
-    size_t combinedStateCount =
-        countStates(ha, a_state_ids) + countStates(hb, b_state_ids) - cpl;
+bool shouldMerge(const NGHolder &ha, const NGHolder &hb, size_t cpl,
+                 const ReportManager *rm, const CompileContext &cc) {
+    size_t combinedStateCount = num_vertices(ha) + num_vertices(hb) - cpl;
+
+    combinedStateCount -= 2 * 2; /* discount accepts from both */
+
+    if (is_triggered(ha)) {
+        /* allow for a state for each top, ignore existing starts */
+        combinedStateCount -= 2; /* for start, startDs */
+        auto tops = getTops(ha);
+        insert(&tops, getTops(hb));
+        combinedStateCount += tops.size();
+    }
 
     if (combinedStateCount > FAST_STATE_LIMIT) {
         // More complex implementability check.
@@ -424,11 +418,13 @@ void buildNfaMergeQueue(const vector<NGHolder *> &cluster,
 
     // First, make sure all holders have numbered states and collect their
     // counts.
-    vector<ue2::unordered_map<NFAVertex, u32>> states_map(cs);
+    vector<ranking_info> states_map;
+    states_map.reserve(cs);
     for (size_t i = 0; i < cs; i++) {
         assert(cluster[i]);
-        NGHolder &g = *(cluster[i]);
-        states_map[i] = numberStates(g);
+        assert(states_map.size() == i);
+        const NGHolder &g = *(cluster[i]);
+        states_map.emplace_back(g);
     }
 
     vector<u16> seen_cpl(cs * cs, 0);
@@ -506,26 +502,25 @@ bool mergeableStarts(const NGHolder &h1, const NGHolder &h2) {
         return false;
     }
 
+    /* TODO: relax top checks if reports match */
+
     // If both graphs have edge (start, accept), the tops must match.
-    auto e1_accept = edge(h1.start, h1.accept, h1);
-    auto e2_accept = edge(h2.start, h2.accept, h2);
-    if (e1_accept.second && e2_accept.second &&
-        h1[e1_accept.first].top != h2[e2_accept.first].top) {
+    NFAEdge e1_accept = edge(h1.start, h1.accept, h1);
+    NFAEdge e2_accept = edge(h2.start, h2.accept, h2);
+    if (e1_accept && e2_accept && h1[e1_accept].tops != h2[e2_accept].tops) {
         return false;
     }
 
     // If both graphs have edge (start, acceptEod), the tops must match.
-    auto e1_eod = edge(h1.start, h1.acceptEod, h1);
-    auto e2_eod = edge(h2.start, h2.acceptEod, h2);
-    if (e1_eod.second && e2_eod.second &&
-        h1[e1_eod.first].top != h2[e2_eod.first].top) {
+    NFAEdge e1_eod = edge(h1.start, h1.acceptEod, h1);
+    NFAEdge e2_eod = edge(h2.start, h2.acceptEod, h2);
+    if (e1_eod && e2_eod && h1[e1_eod].tops != h2[e2_eod].tops) {
         return false;
     }
 
     // If one graph has an edge to accept and the other has an edge to
     // acceptEod, the reports must match for the merge to be safe.
-    if ((e1_accept.second && e2_eod.second) ||
-        (e2_accept.second && e1_eod.second)) {
+    if ((e1_accept && e2_eod) || (e2_accept && e1_eod)) {
         if (h1[h1.start].reports != h2[h2.start].reports) {
             return false;
         }
@@ -535,11 +530,9 @@ bool mergeableStarts(const NGHolder &h1, const NGHolder &h2) {
 }
 
 /** Merge graph \p ga into graph \p gb. Returns false on failure. */
-bool mergeNfaPair(NGHolder &ga, NGHolder &gb, const ReportManager *rm,
+bool mergeNfaPair(const NGHolder &ga, NGHolder &gb, const ReportManager *rm,
                   const CompileContext &cc) {
     assert(ga.kind == gb.kind);
-    auto a_state_ids = numberStates(ga);
-    auto b_state_ids = numberStates(gb);
 
     // Vacuous NFAs require special checks on their starts to ensure that tops
     // match, and that reports match for mixed-accept cases.
@@ -548,29 +541,26 @@ bool mergeNfaPair(NGHolder &ga, NGHolder &gb, const ReportManager *rm,
         return false;
     }
 
-    u32 cpl = commonPrefixLength(ga, a_state_ids, gb, b_state_ids);
-    if (!shouldMerge(gb, b_state_ids, ga, a_state_ids, cpl, rm, cc)) {
+    u32 cpl = commonPrefixLength(ga, gb);
+    if (!shouldMerge(gb, ga, cpl, rm, cc)) {
         return false;
     }
 
     mergeNfaComponent(gb, ga, cpl);
     reduceImplementableGraph(gb, SOM_NONE, rm, cc);
-    b_state_ids = numberStates(gb);
     return true;
 }
 
-/** Merge the group of graphs in \p cluster where possible. The (from, to)
- * mapping of merged graphs is returned in \p merged. */
-void mergeNfaCluster(const vector<NGHolder *> &cluster,
-                     const ReportManager *rm,
-                     map<NGHolder *, NGHolder *> &merged,
-                     const CompileContext &cc) {
+map<NGHolder *, NGHolder *> mergeNfaCluster(const vector<NGHolder *> &cluster,
+                                            const ReportManager *rm,
+                                            const CompileContext &cc) {
+    map<NGHolder *, NGHolder *> merged;
+
     if (cluster.size() < 2) {
-        return;
+        return merged;
     }
 
     DEBUG_PRINTF("new cluster, size %zu\n", cluster.size());
-    merged.clear();
 
     priority_queue<NfaMergeCandidateH> pq;
     buildNfaMergeQueue(cluster, &pq);
@@ -599,6 +589,8 @@ void mergeNfaCluster(const vector<NGHolder *> &cluster,
             }
         }
     }
+
+    return merged;
 }
 
 } // namespace ue2
