@@ -1370,6 +1370,7 @@ TEST(HyperscanArgChecks, AllocScratchBogusScratch) {
     ASSERT_EQ(HS_SUCCESS, err);
     ASSERT_TRUE(db != nullptr);
     hs_scratch_t *blah = (hs_scratch_t *)malloc(100);
+    ASSERT_TRUE(blah != nullptr);
     memset(blah, 0xf0, 100);
     err = hs_alloc_scratch(db, &blah);
     ASSERT_EQ(HS_INVALID, err);
@@ -2034,6 +2035,7 @@ TEST(HyperscanArgChecks, ScratchSizeBadScratch) {
 TEST(HyperscanArgChecks, CloneBadScratch) {
     // Try cloning the scratch
     void *local_garbage = malloc(sizeof(garbage));
+    ASSERT_TRUE(local_garbage != nullptr);
     memcpy(local_garbage, garbage, sizeof(garbage));
     hs_scratch_t *cloned = nullptr;
     hs_scratch_t *scratch = (hs_scratch_t *)local_garbage;
@@ -2052,6 +2054,7 @@ TEST(HyperscanArgChecks, ScanBadScratch) {
     ASSERT_TRUE(db != nullptr);
 
     void *local_garbage = malloc(sizeof(garbage));
+    ASSERT_TRUE(local_garbage != nullptr);
     memcpy(local_garbage, garbage, sizeof(garbage));
 
     hs_scratch_t *scratch = (hs_scratch_t *)local_garbage;
@@ -2072,6 +2075,7 @@ TEST(HyperscanArgChecks, ScanStreamBadScratch) {
     ASSERT_EQ(HS_SUCCESS, err);
     ASSERT_TRUE(db != nullptr);
     void *local_garbage = malloc(sizeof(garbage));
+    ASSERT_TRUE(local_garbage != nullptr);
     memcpy(local_garbage, garbage, sizeof(garbage));
     hs_scratch_t *scratch = (hs_scratch_t *)local_garbage;
 
@@ -2107,6 +2111,7 @@ TEST(HyperscanArgChecks, ResetStreamBadScratch) {
     ASSERT_EQ(HS_SUCCESS, err);
     ASSERT_TRUE(db != nullptr);
     void *local_garbage = malloc(sizeof(garbage));
+    ASSERT_TRUE(local_garbage != nullptr);
     memcpy(local_garbage, garbage, sizeof(garbage));
     hs_scratch_t *scratch = (hs_scratch_t *)local_garbage;
 
@@ -2142,6 +2147,7 @@ TEST(HyperscanArgChecks, ScanVectorBadScratch) {
     ASSERT_EQ(HS_SUCCESS, err);
     ASSERT_TRUE(db != nullptr);
     void *local_garbage = malloc(sizeof(garbage));
+    ASSERT_TRUE(local_garbage != nullptr);
     memcpy(local_garbage, garbage, sizeof(garbage));
     hs_scratch_t *scratch = (hs_scratch_t *)local_garbage;
 
