@@ -126,27 +126,29 @@ TEST(DoubleVermicelli, ExecNoMatch1) {
             const u8 *rv = vermicelliDoubleExec('a', 'b', 0, (u8 *)t1 + i,
                                                 (u8 *)t1 + strlen(t1) - j);
 
-            ASSERT_EQ(((size_t)t1 + strlen(t1) - j  - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1 + strlen(t1) - j), (size_t)rv);
 
             rv = vermicelliDoubleExec('B', 'b', 0, (u8 *)t1 + i,
                                 (u8 *)t1 + strlen(t1) - j);
 
-            ASSERT_EQ(((size_t)t1 + strlen(t1) - j  - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1 + strlen(t1) - j), (size_t)rv);
 
             rv = vermicelliDoubleExec('A', 'B', 1, (u8 *)t1 + i,
                                 (u8 *)t1 + strlen(t1) - j);
 
-            ASSERT_EQ(((size_t)t1 + strlen(t1) - j  - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1 + strlen(t1) - j), (size_t)rv);
 
+            /* partial match */
             rv = vermicelliDoubleExec('b', 'B', 0, (u8 *)t1 + i,
                                 (u8 *)t1 + strlen(t1) - j);
 
-            ASSERT_EQ(((size_t)t1 + strlen(t1) - j  - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1 + strlen(t1) - j - 1), (size_t)rv);
 
+            /* partial match */
             rv = vermicelliDoubleExec('B', 'A', 1, (u8 *)t1 + i,
                                 (u8 *)t1 + strlen(t1) - j);
 
-            ASSERT_EQ(((size_t)t1 + strlen(t1) - j  - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1 + strlen(t1) - j - 1), (size_t)rv);
         }
     }
 }
@@ -353,30 +355,32 @@ TEST(DoubleVermicelliMasked, ExecNoMatch1) {
                                                   t1_raw + i,
                                                   t1_raw + t1.length() - i - j);
 
-            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j - 1), (size_t)rv);
-            rv = vermicelliDoubleMaskedExec('B', 'b', 0xff, CASE_CLEAR,
+            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j), (size_t)rv);
+
+            rv = vermicelliDoubleMaskedExec('B', 'B', 0xff, CASE_CLEAR,
                                             t1_raw + i,
                                             t1_raw + t1.length() - i - j);
 
-            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j  - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j), (size_t)rv);
 
             rv = vermicelliDoubleMaskedExec('A', 'B', CASE_CLEAR, CASE_CLEAR,
                                             t1_raw + i,
                                             t1_raw + t1.length() -i - j);
 
-            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j  - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j), (size_t)rv);
 
-            rv = vermicelliDoubleMaskedExec('b', 'B', CASE_CLEAR, 0xff,
+            /* partial match */
+            rv = vermicelliDoubleMaskedExec('B', 'B', CASE_CLEAR, 0xff,
                                             t1_raw + i,
                                             t1_raw + t1.length() - i - j);
 
-            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j  - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j - 1), (size_t)rv);
 
             rv = vermicelliDoubleMaskedExec('B', 'A', 0xff, 0xff,
                                             t1_raw + i,
                                             t1_raw + t1.length() - i - j);
 
-            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j - 1), (size_t)rv);
+            ASSERT_EQ(((size_t)t1_raw + t1.length() - i - j), (size_t)rv);
         }
     }
 }
